import { useId } from 'react';
import classnames from 'classnames';
import PropTypes from 'prop-types';

import Placeholder from './Placeholder';

import useMountOnObserved from './hooks/useMountOnObserved';

import baseClasses from './styles.module.scss';

const Ad = (props) => {
  const {
    type,
    classes: customClasses,
    observationThreshold,
    rootMargin,
    rootThreshold,
    rootNode,
    alwaysDisplayPlaceholder,
  } = props;

  const {
    container: containerClass,
    root: rootClass,
    disclaimer: disclaimerClass,
  } = customClasses;

  const rId = useId();
  const id = `ad-${rId.split(':')[1]}`;
  const [ref,, mounted] = useMountOnObserved({
    observationThreshold,
    rootMargin,
    rootThreshold,
    rootNode,
    id,
    type,
  });

  return (
    <div
      ref={ref}
      className={classnames(
        baseClasses['PibAd-container'],
        baseClasses['content-group'],
        baseClasses['screenshot-remove'],
        baseClasses[type],
        containerClass,
      )}
    >
      <p className={classnames(
        baseClasses['PibAd-disclaimer'],
        disclaimerClass,
      )}
      >
        Advertisement
      </p>
      <Placeholder
        active={alwaysDisplayPlaceholder || !mounted}
        type={type}
      />
      <div
        id={id}
        className={classnames(
          baseClasses['PibAd-root'],
          baseClasses['ad-slot'],
          baseClasses.flex,
          baseClasses.horizontal,
          rootClass,
        )}
      />
    </div>
  );
};

Ad.defaultProps = {
  type: 'supercube',
  classes: {},
  observationThreshold: 1,
  rootMargin: '700px',
  alwaysDisplayPlaceholder: false,
  rootNode: undefined,
  rootThreshold: 1,
};

Ad.propTypes = {
  /**
   * What type of ad to call for
   */
  type: PropTypes.oneOf([
    'cube',
    'supercube',
    'leaderboard',
    'leaderboard-320',
  ]),
  /**
   * Additional style classes
   */
  classes: PropTypes.shape({
    container: PropTypes.string,
    root: PropTypes.string,
    disclaimer: PropTypes.string,
  }),
  /**
   * How long (in ms) should the ad be in sight before loading
   */
  observationThreshold: PropTypes.number,
  /**
   * The area around the ad which counts as "being in sight"
   */
  rootMargin: PropTypes.string,
  /**
   * The element that is used as the viewport for checking
   * visibility of the target. Must be the ancestor of the
   * target. Defaults to the browser viewport if not specified
   * or if null.
   */
  rootNode: PropTypes.element,
  /**
   * Either a single number or an array of numbers which
   * indicate at what percentage of the target's visibility
   * the observer's callback should be executed.
   */
  rootThreshold: PropTypes.oneOfType([
    PropTypes.number,
    PropTypes.arrayOf(PropTypes.number),
  ]),
  /**
   * An option to keep the placeholder even if the ad is mounted
   */
  alwaysDisplayPlaceholder: PropTypes.bool,
};

export default Ad;
